part of frp;

class StreamMerger{
  
  //--------------------------------------------------------------------------
  //
  //                    Contsructor
  //
  //--------------------------------------------------------------------------
  
  StreamMerger(List<Stream> streams, Function mergeFunc){
    _streams = streams;
    _mergeFunc = mergeFunc;
    
    _bind();
  }
  
  //--------------------------------------------------------------------------
  //
  //                    Variables
  //
  //--------------------------------------------------------------------------
  
  List<Stream> _streams;
  List<StreamRecord> _items;
  HashMap<Stream, StreamRecord> _map = new HashMap();
  Function _mergeFunc;
  StreamController _controller = new StreamController();
  int _nDataCollected = 0;
  
  
  //--------------------------------------------------------------------------
  //
  //                    Methods
  //
  //--------------------------------------------------------------------------
  
  /**
   * Clear recorded info (cache)
   */
  void _clear(){
    _items.forEach((StreamRecord it){
      it.isValid = false;
    });
    _nDataCollected = 0;
  }
  
  /**
   * Pull all the items into a flat list with only theirs values.
   */
  List<Object> _toList(List<StreamRecord> items){
    List<Object> list = new List<Object>();
    items.forEach((StreamRecord it){
      list.add(it.value);
    });
    return list;
  }
  
  /**
   * Bind all the input stream into the merged stream
   */
  void _bind(){
    // Create mapping
    _items = new List<StreamRecord>(_streams.length);
    _map[_streams[0]] = _items[0];
    
    int index = 0;
    _streams.forEach((Stream s){
      // Init entry and set up mapping
      _items[index] = new StreamRecord();
      _map[s] = _items[index++];
      
      // Listen for each stream
      s.listen((data){
        
        StreamRecord it = _map[s];
        it.value = data;
        // Check if data has been collected from each stream
        if(!it.isValid){
          _nDataCollected++;
          it.isValid = true;
          
          // if so, use the merge function to output to the final stream
          if(_nDataCollected == _streams.length){
            var event = _mergeFunc(_toList(_items));
            _controller.add(event);
            // clear all the records
            _clear();
          }
        }
      });  
    });
  }
  
  //--------------------------------------------------------------------------
  //
  //                    Properties
  //
  //--------------------------------------------------------------------------
  Stream get stream => _controller.stream;    
}